function makeEscapeSequence(charCode)
{
	if (charCode == 0x0d)
		return '\\r';
	else if (charCode == 0x0a)
		return '\\n';
	else if (charCode == 0x09)
		return '\\n';
	else
	{
		var hexCharacters = '0123456789abcdef';
		return '\\x' + hexCharacters[charCode>>4] + hexCharacters[charCode & 0xf];
	}
}

function escapeString(str)
//replaces unprintable characters in str with javascript string constant escape sequences
{
	var escapedString = '';
	for (i in str)
	{
		if (str.charCodeAt(i) < 0x20 || str.charCodeAt(i) >= 0x7f)
			escapedString += makeEscapeSequence(str.charCodeAt(i));
		else if (str[i] == '+')
			escapedString += ' ';
		else if (str[i] == '\\')
			escapedString += '\\\\';
		else
			escapedString += str[i];
	}
	return escapedString;
}

function unescapeString(str)
//replaces javascript string constant escape sequences in str with their character values
{
	//this assumes that \uabdc never actually gets used
	try
	{
		return eval("'" + str.replace(/'/g, '\uabdc') + "'").replace(/\uabdc/g, '\'');
	}
	catch (e)
	{
		//the above will fail if the string ends with an unescaped backslash, so handle that case specially
		return eval("'" + str.replace(/'/g, '\uabdc') + "\\'").replace(/\uabdc/g, '\'');
	}
}

function onLoad()
{
	//get the most recent entry in the session history
	var sessionHistory = window.opener.gBrowser.webNavigation.sessionHistory;
	var entry = sessionHistory.getEntryAtIndex(sessionHistory.index, 0);
	entry = entry.QueryInterface(Components.interfaces.nsISHEntry); 

	//create a url object to manipulate the uri with
	var url =
		Components.classes["@mozilla.org/network/standard-url;1"].
		createInstance(Components.interfaces.nsIStandardURL);
	url = url.QueryInterface(Components.interfaces.nsIURL);

	//grab the uri from the session history entry and extract the query from the uri
	url.spec = entry.URI.spec;
	var query = url.query;
	url.query = "";
	var uri = url.spec;

	//grab the referring uri from the session history entry
	var referringUri = entry.referrerURI;
	if (referringUri)
	{
		referringUri = referringUri.QueryInterface(Components.interfaces.nsIURI);
		referringUri = referringUri.spec;
	}
	else
	{
		referringUri = "";
	}

	//update the uri and referringUri textboxes
	document.getElementById("uri-textBox").value = uri;
	document.getElementById("referringUri-textBox").value = referringUri;

	//parse the query string
	if (query!="")
	{
		var pairs = query.split("&");
		for (i in pairs)
		{
			pair = pairs[i].replace(/=/g, escape('=')).replace(escape('='), '=').split("=");
			name = escapeString(unescape(pair[0]));
			value = escapeString(unescape(pair[1]));
			addRowToGetData(name, value);
		}
	}

	//handle the post data
	if (entry.postData)
	{
		var postDataStream = entry.postData;
		postDataStream.QueryInterface(Components.interfaces.nsISeekableStream).seek(0, 0);

		//create an input stream for reading the post data from
		var inputStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
		inputStream.init(entry.postData);
	
		var postData = inputStream.read(0xFFFFFFFF);
		var pairs = postData.split("\r\n\r\n")[1].split("&");
		for (i in pairs)
		{
			pair = pairs[i].replace(/=/g, escape('=')).replace(escape('='), '=').split("=");
			name = escapeString(unescape(pair[0]));
			value = escapeString(unescape(pair[1]));
			addRowToPostData(name, value);
		}
	}
}

function createRow(nameStr, valueStr)
//creates a new listbox row containing textboxes with contents nameStr and valueStr
{
	var nameTextBox = document.createElement("textbox");
	nameTextBox.flex = "1";
	nameTextBox.setAttribute("value", nameStr);

	var nameCell = document.createElement("listcell");
	nameCell.appendChild(nameTextBox);

	var valueTextBox = document.createElement("textbox");
	valueTextBox.flex = "1";
	valueTextBox.setAttribute("value", valueStr);

	var valueCell = document.createElement("listcell");
	valueCell.appendChild(valueTextBox);

	var newListItem = document.createElement("listitem");
	newListItem.appendChild(nameCell);
	newListItem.appendChild(valueCell);

	return newListItem;
}

function addRowToPostData(nameStr, valueStr)
{
	var postDataListBox = document.getElementById("postDataListBox");
	postDataListBox.appendChild(createRow(nameStr, valueStr));
}

function addRowToGetData(nameStr, valueStr)
{
	var getDataListBox = document.getElementById("getDataListBox");
	getDataListBox.appendChild(createRow(nameStr, valueStr));
}

function onPostDataAddRowClick()
{
	addRowToPostData("", "");
}

function onPostDataDeleteRowClick()
{
	var listBox = document.getElementById("postDataListBox");
	var selectedItem = listBox.selectedItem;
	listBox.removeChild(selectedItem);
}

function onGetDataAddRowClick()
{
	addRowToGetData("", "");
}

function onGetDataDeleteRowClick()
{
	var listBox = document.getElementById("getDataListBox");
	var selectedItem = listBox.selectedItem;
	listBox.removeChild(selectedItem);
}

function onGetDataListBoxSelect()
{
	var listBox = document.getElementById("getDataListBox");
	var deleteRowButton = document.getElementById("getData-deleteRowButton");
	deleteRowButton.disabled=(listBox.selectedCount==0);
}

function onPostDataListBoxSelect()
{
	var listBox = document.getElementById("postDataListBox");
	var deleteRowButton = document.getElementById("postData-deleteRowButton");
	deleteRowButton.disabled=(listBox.selectedCount==0);
}

function urlEncodeListBox(listBox)
{
	var listItems = listBox.getElementsByTagName("listitem");
	var pairs = "";
	for (i=0; i<listItems.length; i++)
	{
		var listItem = listItems[i];
		name = listItem.childNodes[0].firstChild.value;
		value = listItem.childNodes[1].firstChild.value;

		var pair = escape(unescapeString(name))+"="+escape(unescapeString(value));
		if (pairs!="")
			pairs+="&";
		pairs += pair;
	}

	return pairs;
}

function onSubmit()
{
	var getData = urlEncodeListBox(document.getElementById("getDataListBox"));

	var postData = urlEncodeListBox( document.getElementById("postDataListBox"));
	var postDataStream;
	if (postData!="")
	{
		var postDataHeaders = "Content-Type: application/x-www-form-urlencoded\r\nContent-Length: "+postData.length+"\r\n\r\n";
		postDataStream = Components.classes["@mozilla.org/io/string-input-stream;1"].createInstance(Components.interfaces.nsIStringInputStream);
		postDataStream.setData(postDataHeaders + postData, -1);
	}
	else
		postDataStream = null;

	var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIStandardURL);
	uri = uri.QueryInterface(Components.interfaces.nsIURL); 
	uri.spec = document.getElementById("uri-textBox").value;
	uri.query = getData;
	
	var referringUri = document.getElementById("referringUri-textBox").value;
	if (referringUri=="")
		referringUri = null;
	else
	{
		var referringUriStr = referringUri;
		referringUri = Components.classes["@mozilla.org/network/simple-uri;1"].createInstance(Components.interfaces.nsIURI);
		try
		{
			referringUri.spec = referringUriStr;
		}
		catch (e)
		{
			alert('Referring URI \''+referringUriStr+'\' is not a valid URI.');
			throw e;
		}
	}

	var webNavigation = window.opener.gBrowser.webNavigation;

	webNavigation.loadURI(uri.spec, webNavigation.LOAD_FLAGS_NONE, referringUri, postDataStream, null);
}

function onTabSwitch()
{
	var selectedIndex = document.getElementById("tabPanels").selectedIndex;

	if (selectedIndex == 1)
	{
		document.getElementById("postData-addRowButton").removeAttribute("accesskey");
		document.getElementById("postData-deleteRowButton").removeAttribute("accesskey");
		document.getElementById("getData-addRowButton").setAttribute("accesskey", "A");
		document.getElementById("getData-deleteRowButton").setAttribute("accesskey", "D");
	}
	else if (selectedIndex == 2)
	{
		document.getElementById("getData-addRowButton").removeAttribute("accesskey");
		document.getElementById("getData-deleteRowButton").removeAttribute("accesskey");
		document.getElementById("postData-addRowButton").setAttribute("accesskey", "A");
		document.getElementById("postData-deleteRowButton").setAttribute("accesskey", "D");
	}
}
